#include <Stepper.h>
#include "RTClib.h"
RTC_DS1307 rtc;

// Call for steppers
#define oneRotation 2038 // the number of steps in one revolution of 28BYJ-48 stepper motor
Stepper numberOfDay(oneRotation, 2, 4, 3, 5);
Stepper nameOfMonth(oneRotation, 6, 8, 7, 9);
Stepper nameOfDay(oneRotation, 10, 12, 11, 13);

// Location of the hall effect sensors used to find the home position of the rings.
#define NUMBER_OF_DAY_SENSOR A0
#define NAME_OF_MONTH_SENSOR A1
#define NAME_OF_DAY_SENSOR A2

// Variables used to keep track of the current day.
uint8_t thisNumberDay = 0;
uint8_t thisMonth = 0;
uint8_t thisNameDay = 0;
uint8_t nextNumberDay = 0;
uint8_t nextMonth = 0;
uint8_t nextNameDay = 0;

// Constants used to advance each ring one day/month.
const int NUMBER_OF_DAY_TO_STEP = -1661;//1108 steps per day
const int NAME_OF_MONTH_TO_STEP = -2946;//1965 steps per month
const int NAME_OF_DAY_TO_STEP = -2745;//1831 steps per day
const int COUNTERCLOCKWISE = -1;

// Update these constants to adust the position of the rings at the reset position.
// Negative numbers will advance the rings forward (counterclockwise) positive will reverse the rings.
const int NUMBER_OF_DAY_RESET_ADJUST = -130;
const int NAME_OF_MONTH_RESET_ADJUST = -60;
const int NAME_OF_DAY_RESET_ADJUST = -35;

// Bit used to test if the ring has been reset to home.
bool NUMBER_OF_DAY_RESET = LOW;
bool NAME_OF_MONTH_RESET = LOW;
bool NAME_OF_DAY_RESET = LOW;

void setup() {
  Serial.begin(9600);
  rtc.begin();

  // Setup real time clock and reset hall effect sensors.
  pinMode(NUMBER_OF_DAY_SENSOR, INPUT_PULLUP);
  pinMode(NAME_OF_MONTH_SENSOR, INPUT_PULLUP);
  pinMode(NAME_OF_DAY_SENSOR, INPUT_PULLUP);


  // Uncomment line below to set time.
  // rtc.adjust(DateTime(2022, 1, 4, 23, 55, 0));
  // rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));

  // Get current date
  DateTime now = rtc.now();
  thisNumberDay = now.day();
  thisMonth =  now.month();
  thisNameDay = now.dayOfTheWeek();

  Serial.print("Date: ");
  Serial.print(thisNumberDay);
  Serial.print(" ");
  Serial.print(thisMonth);
  Serial.print(" ");
  Serial.println(thisNameDay);

  // Set top speed of stepper motors.
  numberOfDay.setSpeed(15);
  nameOfMonth.setSpeed(15);
  nameOfDay.setSpeed(15);

  // Loop to move any ring off of the hall effect sensor if it is currently tripped.
  while (digitalRead(NUMBER_OF_DAY_SENSOR) == LOW ||  digitalRead(NAME_OF_MONTH_SENSOR) == LOW || digitalRead(NAME_OF_DAY_SENSOR) == LOW)
  {
    if (digitalRead(NUMBER_OF_DAY_SENSOR) == LOW )
    {
      numberOfDay.step(COUNTERCLOCKWISE);
    }

    else
    {
      delay(1);
    }

    if (digitalRead(NAME_OF_MONTH_SENSOR) == LOW)
    {
      nameOfMonth.step(COUNTERCLOCKWISE);
    }

    else
    {
      delay(1);
    }

    if (digitalRead(NAME_OF_DAY_SENSOR) == LOW)
    {
      nameOfDay.step(COUNTERCLOCKWISE);
    }

    else
    {
      delay(1);
    }
  }

  // Loop until all rings are at the reset position.
  while (digitalRead(NUMBER_OF_DAY_SENSOR) != LOW || digitalRead(NAME_OF_MONTH_SENSOR) != LOW || digitalRead(NAME_OF_DAY_SENSOR) != LOW)
  {
    if (digitalRead(NUMBER_OF_DAY_SENSOR) != LOW )
    {
      numberOfDay.step(COUNTERCLOCKWISE);
    }

    else
    {
      delay(1);
    }

    if (digitalRead(NAME_OF_MONTH_SENSOR) != LOW )
    {
      nameOfMonth.step(COUNTERCLOCKWISE);
    }

    else
    {
      delay(1);
    }

    if (digitalRead(NAME_OF_DAY_SENSOR) != LOW)
    {
      nameOfDay.step(COUNTERCLOCKWISE);
    }

    else
    {
      delay(1);
    }
  }

  //  Use these functions to carefully adjust position of rings after it's been reset.
  numberOfDay.step(NUMBER_OF_DAY_RESET_ADJUST);
  nameOfMonth.step(NAME_OF_MONTH_RESET_ADJUST);
  nameOfDay.step(NAME_OF_DAY_RESET_ADJUST);

  // Move each ring to the correct number, month, and day of week
  for (int i = 1; i < thisNumberDay; i++)
  {
    numberOfDay.step(NUMBER_OF_DAY_TO_STEP);
  }

  for (int i = 1; i < thisMonth; i++)
  {
    nameOfMonth.step(NAME_OF_MONTH_TO_STEP);
  }

  for (int i = 0; i < thisNameDay; i++)
  {
    nameOfDay.step(NAME_OF_DAY_TO_STEP);
  }

}

void loop() {

  // Check the current day number
  DateTime now = rtc.now();
  nextNumberDay = now.day();


  // If the current checked day is different than it was last time it was checked, move the rings to the next date.
  if (thisNumberDay != nextNumberDay)
  {
    // Get the next month and day of week.
    nextMonth =  now.month();
    nextNameDay = now.dayOfTheWeek();

    // If the number of the day is 1 (and it's not currently at 1) reset to 1
    if (nextNumberDay == 1 && thisNumberDay >= 28)
    {
      while (digitalRead(NUMBER_OF_DAY_SENSOR) != LOW)
      {
        numberOfDay.step(COUNTERCLOCKWISE);
        NUMBER_OF_DAY_RESET = HIGH;
      }
      numberOfDay.step(NUMBER_OF_DAY_RESET_ADJUST);
    }

    // If the month is Jan and was Dec, rest to Jan.
    if (nextMonth == 1 && thisMonth == 12)
    {
      while (digitalRead(NAME_OF_MONTH_SENSOR) != LOW)
      {
        nameOfMonth.step(COUNTERCLOCKWISE);
        NAME_OF_MONTH_RESET = HIGH;
      }
      nameOfMonth.step(NAME_OF_MONTH_RESET_ADJUST);
    }

    // If the day is Sun but was Sat, reset to Sun.
    if (nextNameDay == 0 && thisNameDay == 6)
    {
      while (digitalRead(NAME_OF_DAY_SENSOR) != LOW)
      {
        nameOfDay.step(COUNTERCLOCKWISE);
        NAME_OF_DAY_RESET = HIGH;
      }
      nameOfDay.step(NAME_OF_DAY_RESET_ADJUST);
    }

    // If the ring wasn't reset to the beginning and needs to change, step to the next position
    if (NUMBER_OF_DAY_RESET == LOW && thisNumberDay != nextNumberDay)
    {
      numberOfDay.step(NUMBER_OF_DAY_TO_STEP);
    }

    if (NAME_OF_MONTH_RESET == LOW && thisMonth != nextMonth)
    {
      nameOfMonth.step(NAME_OF_MONTH_TO_STEP);
    }

    if (NAME_OF_DAY_RESET == LOW && thisNameDay != nextNameDay)
    {
      nameOfDay.step(NAME_OF_DAY_TO_STEP);
    }

    // reset check values to test each loop
    thisNumberDay = nextNumberDay;
    thisMonth = nextMonth;
    thisNameDay = nextNameDay;
    NUMBER_OF_DAY_RESET = LOW;
    NAME_OF_MONTH_RESET = LOW;
    NAME_OF_DAY_RESET = LOW;
  }

  delay(60000);
}
